#ifndef __Alibava_AsciiRoot_h__
#define __Alibava_AsciiRoot_h__

#include <vector>
#include <Data.h>
#include <Hit.h>
#include <ctime>

/** This is the class that reads the data files
 */
class AsciiRoot
{
    public:
        typedef std::vector< Hit> HitList;
    private:
        std::ifstream *ifile;
        unsigned int data_start;
        int _type;
        time_t _t0;
        int _npoints;
        int _from;
        int _to;
        int _step;
        int _nevts;
        double _seedcut;
        double _neighcut;
        double _ped[256];
        double _noise[256];
        double _signal[256];
        double _sn[256];
        double _cmmd[2];
        double _cnoise[2];
        double _gain[256];
        double _average_gain;
        bool   _mask[256];
        HitList _hits;

        EventData _data;
    public:
        AsciiRoot(const char *nam, const char *pedfile=0, const char *gainfile=0);
        ~AsciiRoot();

        bool valid() const
        {
            return (ifile!=0);
        }

        void open(const char *name);
        void close();
        void rewind();
        int read_event();

        int type() const
        {
            return _type;
        }
        char *date() const
        {
            return ctime(&_t0);
        }
        double ped(int i) const
        {
            return _ped[i]/_gain[i];
        }
        double noise(int i) const
        {
            return _noise[i]/_gain[i];
        }
        double signal(int i) const
        {
            return _signal[i]/_gain[i];
        }
        
        double sn(int i) const
        {
            return _sn[i];
        }
        
        double get_cmmd(int i) const
        {
            return _cmmd[i];
        }
        
        double get_cnoise(int i) const
        {
            return _cnoise[i];
        }
        
        unsigned short data(int i) const
        {
            return _data.data[i];
        }
        double value() const
        {
            return _data.value;
        }
        double time() const;
        double temp() const;
        int npts() const
        {
            return _npoints;
        }
        int from() const
        {
            return _from;
        }
        int to() const
        {
            return _to;
        }
        int step() const
        {
            return _step;
        }
        int nevts() const
        {
            return _step;
        }

        void add_hit(const Hit &h)
        {
            _hits.push_back(h);
        }
        HitList::iterator begin()
        {
            return _hits.begin();
        }
        HitList::iterator end()
        {
            return _hits.end();
        }
        int nhits() const
        {
            return _hits.size();
        }
        bool empty() const
        {
            return _hits.empty();
        }
        const Hit &hit(int i) const
        {
            return _hits[i];
        }
        void clear()
        {
            _hits.clear();
        }
        
        double get_gain(int i) const 
        {
            return _gain[i];
        }
        double gain() const 
        {
            return _average_gain;
        }

        double seed_cut() const 
        {
            return _seedcut;
        }
        double neigh_cut() const
        {
            return _neighcut;
        }
        void set_cuts(double s, double n)
        {
            _seedcut = s;
            _neighcut = n;
        }
        TH1 *show_pedestals();
        TH1 *show_noise();
        TH2 *compute_pedestals(int mxevts=-1, bool do_cmmd=true);
        void compute_pedestals_fast(int mxevts = -1, double ped_weight=0.01, double noise_weight=0.001);
        
        void process_event(bool do_cmmd=true);
        void find_clusters(double polarity=0., int ichip=-1);
        void save_pedestals(const char *fnam);
        void load_pedestals(const char *fnam);
        void load_gain(const char *fnam);
        void spy_data(bool with_signal=false, int nevt=1);
        void common_mode();

};


#endif
